from datetime import datetime
from enum import Enum
import socket

from influxdb import InfluxDBClient


class ConstantEntry(Enum):
    
    LOAD = "load_constants"
    SAVE = "save_constants"
    

class InfluxLogger:
    
    def __init__(self, detector=None, instrument=None, notebook=None,
                 run=None, proposal=None, mem_cells=None):
        self.detector = detector
        self.instrument = instrument
        self.notebook = notebook
        self.run = run
        self.proposal = proposal
        self.mem_cells = mem_cells
        self.entries = []
        
    def constants_entry(self, optype, qm, constant, success, origin, loadtime, reason=""):
        self.entries.append( {
            "measurement": optype.value,
            "tags": {
                "host": socket.gethostname(),
                "run": self.run,
                "proposal": self.proposal,
                "mem_cells": self.mem_cells,
                "constant": constant,
                "module": qm,
                "origin": origin,
                "detector": self.detector,
                "instrument": self.instrument,
            },
            "time": datetime.utcnow().isoformat(),
            "fields": {
                "success": success,
                "reason": reason,
                "load_time": loadtime,
            }
        })
        

    def runtime_entry(self, qm, sequence, filesize, chunksize, 
                      total_sequences, sequences_per_module,
                      success, runtime, reason=""):
        self.entries.append( {
            "measurement": "run_correction",
            "tags": {
                "host": socket.gethostname(),
                "run": self.run,
                "proposal": self.proposal,
                "mem_cells": self.mem_cells,
                "sequence": sequence,
                "module": qm,
                "chunksize": chunksize,
                "detector": self.detector,
                "instrument": self.instrument,
                "filesize": filesize,
                "total_sequences": total_sequences,
                "sequences_module": sequences_per_module,
            },
            "time": datetime.utcnow().isoformat(),
            "fields": {
                "success": success,
                "reason": reason,
                "runtime": runtime,      
                               
            }
        })
    
    def runtime_summary_entry(self, success, runtime, filesize=None, chunksize=None,
                              total_sequences=None, sequences_per_module=None,
                              reason=""):
        self.entries.append( {
            "measurement": "run_correction",
            "tags": {
                "host": socket.gethostname(),
                "run": self.run,
                "proposal": self.proposal,
                "mem_cells": self.mem_cells,        
                "chunksize": chunksize,
                "detector": self.detector,
                "instrument": self.instrument,
            },
            "time": datetime.utcnow().isoformat(),
            "fields": {
                "success": success,
                "reason": reason,
                "runtime": runtime, 
                "filesize": filesize,
                "total_sequences": total_sequences,
            }
        })
        
    def send(self):
        try:
            client = InfluxDBClient('exflqr18318', 8086, 'root', 'root', 'calstats')
            client.write_points(self.entries)
            self.entries = []
        except Exception as e:
            print("Could not send to influx DB: {}".format(e))
            